import gtk
import gobject
import pango

import os
ICON_DIR = os.path.join(os.path.dirname(__file__), "icons")

from xesam.ontology import *

class OntologyTreeModel (gtk.TreeStore):
	def __init__ (self):
		gtk.TreeStore.__init__ (self, gobject.TYPE_PYOBJECT)
	
	def import_roots (self, roots):
		for root in roots:
			iter = self.append(None, [root])
			for child in root.get_children():
				self.import_item (iter, child)
				
	def import_item (self, iter, item):
		nest = self.append (iter, [item])
		
		# Alphabetic sort of children
		children = list(item.get_children())
		children.sort (lambda x, y : cmp (x.get_id(), y.get_id()))
		for child in children:
			self.import_item (nest, child)
		if isinstance (item, FieldMapper):
			# Alphabetic sort of mapped fields
			fields = list(item.get_fields())
			fields.sort (lambda x, y : cmp (x.get_id(), y.get_id()))
			for field in fields:
				self.import_item (nest,field)
		
		
class OntologyTreeView (gtk.TreeView):
	PIXBUFS = \
	{
		Item.TYPE_CONTENT : "content",
		Item.TYPE_FIELD : "field",
		Item.TYPE_SOURCE : "source"
	}

	__gsignals__ = {
		# Emitted when an ontology.Item is selected
		"item-selected" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_PYOBJECT])
	}
	
	def __init__ (self, model):
		gtk.TreeView.__init__ (self, model)
				
		icon = gtk.CellRendererPixbuf ()
		title = gtk.CellRendererText ()
		title.set_property ("ellipsize", pango.ELLIPSIZE_END)
		title.set_property ("width-chars", 100) #FIXME: Pick width according to screen size
		items = gtk.TreeViewColumn ("Items")
		items.pack_start (icon)
		items.pack_start (title)
		items.set_cell_data_func(title, self._get_title_for_cell)			
		items.set_cell_data_func(icon, self._get_icon_for_cell)
		self.append_column (items)
		
		self.connect ("cursor-changed", lambda view : self._on_cursor_changed())
		self.set_property ("headers-visible", False)
				
		self.set_enable_search (True)
		self.set_property ("enable-tree-lines", True)
		self.set_property ("level-indentation", 10)
		
		# Prepare a buffer for the needed icons
		self._pixbufs = {}
		self._load_pixbufs()
	
	def _get_title_for_cell (self, column, cell, model, iter, data=None):
		item = model[iter][0]
		if item.is_external():
			cell.set_property ("markup", item.get_id() + " (external)")
		else:
			cell.set_property ("markup", item.get_id())
		
	def _get_icon_for_cell (self, column, cell, model, iter, data=None):
		item = model[iter][0]
		cell.set_property ("pixbuf", self._pixbufs[item.get_type()])
	
	def _on_cursor_changed (self):
		model, iter = self.get_selection().get_selected()
		self.emit ("item-selected", model[iter][0])
		
	def _load_pixbufs (self):
		for icon in self.PIXBUFS.iterkeys() :
			if not self._pixbufs.has_key(icon):
				self._pixbufs[icon] = gtk.gdk.pixbuf_new_from_file(os.path.join(ICON_DIR, self.PIXBUFS[icon]+ ".png"))
				xesam.debug ("Loaded icon " + self.PIXBUFS[icon]+".png")

class ItemView (gtk.TreeView):
	[
		COL_DESC,
		COL_DATA,
		COL_SETTER
	] = range (3)
	
	def __init__ (self):
		gtk.TreeView.__init__ (self)		
		model =	gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
		self.set_model (model)
		
		desc_r = gtk.CellRendererText ()
		desc_r.set_property ("weight", 800)
		desc_r.set_property ("xalign", 1)
		desc_r.set_property ("yalign", 0)

		data_r = gtk.CellRendererText ()
		data_r.set_property ("wrap-width", 200)
		data_r.connect ("edited", self._edited)
				
		desc = gtk.TreeViewColumn ("Property")
		desc.pack_start (desc_r)
		desc.set_cell_data_func(desc_r, self._get_description_for_cell)

		data = gtk.TreeViewColumn ("Data")
		data.pack_start (data_r)
		data.set_cell_data_func(data_r, self._get_data_for_cell)
		
		self.append_column (desc)
		self.append_column (data)		
		self.set_property ("headers-visible", False)				
		self.set_enable_search (True)
		
		self._item = None
	
	def _get_description_for_cell (self, column, cell, model, iter, data=None):
		desc = model[iter][self.COL_DESC]
		cell.set_property ("markup", desc)
		
	def _get_data_for_cell (self, column, cell, model, iter, data=None):
		data = model[iter][self.COL_DATA]
		setter = model[iter][self.COL_SETTER]
		cell.set_property ("markup", data)
		
		if setter is None:
			cell.set_property ("editable", False)
		else:
			cell.set_property ("editable", True)
		
	
	def set_item (self, item):
		self._item = item
		model = self.get_model()
		model.clear()
		
		child_names = list(item.get_all_child_names())
		child_names.sort ()
		
		model.append (None, ["Id", item.get_id(), item.set_id])
		model.append (None, ["Url", item["Url"], lambda x : self._item_prop("Url", x)])
		model.append (None, ["Type", item.get_type(), item.set_type])
		model.append (None, ["Comment", item["Comment"], None])
		model.append (None, ["Parents", ", ".join([parent.get_id() for parent in item.get_parents()]), None])
		model.append (None, ["Children", "\n".join(child_names), None])
		
		
		if isinstance (item, FieldMapper):
			fields = [field.get_id() for field in item.get_fields()]
			all_fields = [field.get_id() for field in item.get_all_fields()]
			
			fields.sort()
			all_fields.sort()
			
			model.append (None, ["Fields", "\n".join(fields), None])
			model.append (None, ["All Fields", "\n".join(all_fields), None])
		elif isinstance (item, Field):
			card = item["Cardinality"]
			if item.get_data_type() is None:
				model.append (None, ["Data Type", "This is an abstract field and does not hold a value", None])
			elif not card :
				model.append (None, ["Data Type", "List of " + item.get_data_type() + "s", item.set_data_type])
			elif int(card) == 1:
				model.append (None, ["Data Type", item.get_data_type(), item.set_data_type])
			else:
				xesam.error("Field cardinalities should be 1 or unset (for lists). Found %s for field %s"
						% (card, item.get_id()))
	
	def _set_item_prop (self, prop, val):
		self._item[prop] = val
	
	def _edited (self, cell, path, text):
		model = self.get_model()
		setter = model[model.get_iter(path)][self.COL_SETTER]
		setter(text)
		
		# Force ui update
		self.set_item (self._item)
